package lc.syntax.tree;

import java.util.OptionalInt;
import java.util.stream.Stream;

public record Application(Term left, Term right) implements Term {

    @Override
    public Stream<String> free() {
        return Stream.concat(this.left.free(), this.right.free());
    }

    @Override
    public Term substitute(final String name, final Term term) {
        return new Application(
            this.left.substitute(name, term),
            this.right.substitute(name, term)
        );
    }

    @Override
    public String show(final boolean numbers) {
        final StringBuilder builder = new StringBuilder();

        if (left instanceof Abstraction) {
            builder.append('(');
            builder.append(left.show(numbers));
            builder.append(')');
        } else {
            builder.append(left.show(numbers));
        }

        builder.append(' ');

        if (right instanceof Application || (right instanceof Abstraction && right.toNumber().isEmpty())) {
            builder.append('(');
            builder.append(right.show(numbers));
            builder.append(')');
        } else {
            builder.append(right.show(numbers));
        }

        return builder.toString();
    }

    @Override
    public OptionalInt toNumber() {
        return OptionalInt.empty();
    }
}
